#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "pl.h"
#include "utils.h"


void version(void)
{
	printf("nagtail, v" VERSION " (C) 2006 by folkert@vanheusden.com\n");
}

void usage(void)
{
	version();

	printf("\n");

	printf("-f     path to status.log ('status_file' parameter in nagios.cfg)\n");
	printf("-F     host:port for retrieving status.log\n");
	printf("-x     status.log is in nagios 1.0 format\n");
	printf("-X     status.log is in nagios 2.0 format\n");
	printf("-h     show only state type 'HARD' (default)\n");
	printf("-H     show also state type 'SOFT'\n");
	printf("-i     check interval (in seconds - default 60)\n");
}

int main(int argc, char *argv[])
{
	struct stats *prev = NULL;
	int n_prev = 0;
	int c;
	char *statuslog = "/usr/local/nagios/var/status.log";
	int check_interval = 60;
	char verbose = 0;
	char is_20_format = 1;
	char is_file = 1;
	char hard_only = 1;
	char one_line = 0;

	while((c = getopt(argc, argv, "1xXF:hHf:i:vV")) != -1)
	{
		switch(c)
		{
			case '1':
				one_line = 1;
				break;

			case 'h':
				hard_only = 1;
				break;

			case 'H':
				hard_only = 0;
				break;

			case 'x':
				is_20_format = 0;
				break;

			case 'X':
				is_20_format = 1;
				break;

			case 'f':
				statuslog = optarg;
				is_file = 1;
				break;

			case 'F':
				statuslog = optarg;
				is_file = 0;
				break;

			case 'i':
				check_interval = atoi(optarg);
				break;

			case 'v':
				verbose = 1;
				break;

			case 'V':
				version();
				return 0;

			default:
				usage();
				return 0;
		}
	}

	for(;;)
	{
		struct stats *cur = NULL;
		int n_cur = 0;
		int loop;
		int fd;

		if (verbose)
		{
			time_t now = time(NULL);

			printf("Checking @ %s", ctime(&now));
		}

		if (is_file == 1)     /* file */
			fd = open64(statuslog, O_RDONLY);
		else
			fd = connect_to(statuslog);

		if (fd != -1)
		{
			if (is_20_format)
				parse_2_0_statuslog(fd, &cur, &n_cur);
			else
				parse_1_0_statuslog(fd, &cur, &n_cur);

			close(fd);

			for(loop=0; loop<n_cur; loop++)
			{
				char show = 0;
				int prev_index = -1;

				if (prev)
					prev_index = find_index_by_host_and_service(prev, n_prev, cur[loop].host_name, cur[loop].service_description);

				if (prev_index == -1)
				{
					if (prev == NULL)
					{
						if (should_i_show_entry(cur, n_cur, loop, 0, 0, 0, 1))
							show = 1;
					}
					else
					{
						show = 1;
					}
				}
				else
				{
					if (hard_only)
					{
						if (prev[prev_index].state_type == ST_HARD && cur[loop].state_type == ST_HARD)
						{
							if (prev[prev_index].current_state != cur[loop].current_state)
								show = 1;
						}
						else if (prev[prev_index].state_type == ST_SOFT && cur[loop].state_type == ST_HARD)
						{
							if (cur[loop].current_state != 0)
								show = 1;
						}
					}
					else
					{
						if (prev[prev_index].current_state != cur[loop].current_state)
							show = 1;
					}
				}

				if (show)
				{
					struct tm *ptm = localtime(&cur[loop].last_state_change);
					char *state_str[4] = { " OK ", "WARN", "CRIT", " ?? " };
					char *format;

					if (one_line)
						format = "%04d/%02d/%02d %02d:%02d %s %s %s, %s\n";
					else
						format = "%04d/%02d/%02d %02d:%02d %s %s %s\n%s\n";

					printf(format,
							ptm -> tm_year + 1900, ptm -> tm_mon + 1, ptm -> tm_mday, ptm -> tm_hour, ptm -> tm_min,
							state_str[cur[loop].current_state],
							cur[loop].host_name?cur[loop].host_name:"",
							cur[loop].service_description?cur[loop].service_description:"",
							cur[loop].plugin_output?cur[loop].plugin_output:"");
				}
			}

			fflush(NULL);
		}

		free_stats_array(prev, n_prev);
		prev = cur;
		n_prev = n_cur;

		sleep(check_interval);
	}


	return 0;
}
